# Copyright (c) 2022 Phase Advanced Sensor Systems, Inc.
from psdb.devices import Device, Reg8S, Reg32S, AReg32


class OCTOSPI(Device):
    '''
    Driver for the STM32 octo-SPI peripheral found on many MCUs (OCTOSPI).
    '''
    REGS = [AReg32('CR',        0x000, [('EN',          0),
                                        ('ABORT',       1),
                                        ('DMAEN',       2),
                                        ('TCEN',        3),
                                        ('DMM',         6),
                                        ('MSEL',        7),
                                        ('FTHRES',      8,  12),
                                        ('TEIE',        16),
                                        ('TCIE',        17),
                                        ('FTIE',        18),
                                        ('SMIE',        19),
                                        ('TOIE',        20),
                                        ('APMS',        22),
                                        ('PMM',         23),
                                        ('FMODE',       28,  29),
                                        ]),
            AReg32('DCR1',      0x008, [('CKMODE',      0),
                                        ('FRCK',        1),
                                        ('DLYBYP',      3),
                                        ('CSHT',        8,  13),
                                        ('DEVSIZE',     16, 20),
                                        ('MTYP',        24, 26)
                                        ]),
            AReg32('DCR2',      0x00C, [('PRESCALER',   0,  7),
                                        ('WRAPSIZE',    16, 18),
                                        ]),
            AReg32('DCR3',      0x010, [('MAXTRAN',     0,  7),
                                        ('CSBOUND',     16, 20),
                                        ]),
            AReg32('DCR4',      0x014, [('REFRESH',     0,  31),
                                        ]),
            AReg32('SR',        0x020, [('TEF',         0),
                                        ('TCF',         1),
                                        ('FTF',         2),
                                        ('SMF',         3),
                                        ('TOF',         4),
                                        ('BUSY',        5),
                                        ('FLEVEL',      8,  13),
                                        ]),
            AReg32('FCR',       0x024, [('CTEF',        0),
                                        ('CTCF',        1),
                                        ('CSMF',        3),
                                        ('CTOF',        4),
                                        ]),
            AReg32('DLR',       0x040, [('DL',          0,  31),
                                        ]),
            AReg32('AR',        0x048, [('ADDRESS',     0,  31),
                                        ]),
            Reg8S('DR8',        0x050),
            Reg32S('DR32',      0x050),
            AReg32('PSMKR',     0x080, [('MASK',        0,  31),
                                        ]),
            AReg32('PSMAR',     0x088, [('MATCH',       0,  31),
                                        ]),
            AReg32('PIR',       0x090, [('INTERVAL',    0,  15),
                                        ]),
            AReg32('CCR',       0x100, [('IMODE',       0,  2),
                                        ('IDTR',        3),
                                        ('ISIZE',       4,  5),
                                        ('AMODE',       8,  10),
                                        ('ADDTR',       11),
                                        ('ADSIZE',      12, 13),
                                        ('ABMODE',      16, 18),
                                        ('ABDTR',       19),
                                        ('ABSIZE',      20, 21),
                                        ('DMODE',       24, 26),
                                        ('DDTR',        27),
                                        ('DQSE',        29),
                                        ('SIOO',        31),
                                        ]),
            AReg32('TCR',       0x108, [('DCYC',        0,  4),
                                        ]),
            AReg32('IR',        0x110, [('INSTRUCTION', 0,  31),
                                        ]),
            AReg32('ABR',       0x120, [('ALTERNATE',   0,  31),
                                        ]),
            AReg32('LPTR',      0x130, [('TIMEOUT',     0,  15),
                                        ]),
            AReg32('WPCCR',     0x140, [('IMODE',       0,  2),
                                        ('IDTR',        3),
                                        ('ISIZE',       4,  5),
                                        ('AMODE',       8,  10),
                                        ('ADDTR',       11),
                                        ('ADSIZE',      12, 13),
                                        ('ABMODE',      16, 18),
                                        ('ABDTR',       19),
                                        ('ABSIZE',      20, 21),
                                        ('DMODE',       24, 26),
                                        ('DDTR',        27),
                                        ('DQSE',        29),
                                        ]),
            AReg32('WPTCR',     0x148, [('DCYC',        0,  4),
                                        ('DHQC',        28),
                                        ('SSHIFT',      30),
                                        ]),
            AReg32('WPIR',      0x150, [('INSTRUCTION', 0,  31),
                                        ]),
            AReg32('WPABR',     0x160, [('ALTERNATE',   0,  31),
                                        ]),
            AReg32('WCCR',      0x180, [('IMODE',       0,  2),
                                        ('IDTR',        3),
                                        ('ISIZE',       4,  5),
                                        ('AMODE',       8,  10),
                                        ('ADDTR',       11),
                                        ('ADSIZE',      12, 13),
                                        ('ABMODE',      16, 18),
                                        ('ABDTR',       19),
                                        ('ABSIZE',      20, 21),
                                        ('DMODE',       24, 26),
                                        ('DDTR',        27),
                                        ('DQSE',        29),
                                        ]),
            AReg32('WTCR',      0x188, [('DCYC',        0,  4),
                                        ]),
            AReg32('WIR',       0x190, [('INSTRUCTION', 0,  31),
                                        ]),
            AReg32('WABR',      0x1A0, [('ALTERNATE',   0,  31),
                                        ]),
            AReg32('HLCR',      0x200, [('LM',          0),
                                        ('WZL',         1),
                                        ('TACC',        8,  15),
                                        ('TRWR',        16, 23),
                                        ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super().__init__(target, ap, addr, name, OCTOSPI.REGS, **kwargs)
