# Copyright (c) 2024-2025 Phase Advanced Sensor Systems, Inc.
from ..device import Device, Reg32, AReg32


ENABLE_BITS = {
    # AHB3
    'MDMA'              : (0xD4,  0),
    'DMA2D'             : (0xD4,  4),
    'FMC'               : (0xD4, 12),
    'OCTOSPI1'          : (0xD4, 14),
    'SDMMC1'            : (0xD4, 16),
    'OCTOSPI2'          : (0xD4, 19),
    'IOMNGR'            : (0xD4, 21),
    'OTFD1EN'           : (0xD4, 22),
    'OTFD2EN'           : (0xD4, 23),

    # AHB1
    'DMA1'              : (0xD8,  0),
    'DMA2'              : (0xD8,  1),
    'ADC12'             : (0xD8,  5),
    'ETH1MAC'           : (0xD8, 15),
    'ETH1TX'            : (0xD8, 16),
    'ETH1RX'            : (0xD8, 17),
    'USB1OTGHS'         : (0xD8, 25),
    'USB1OTGHSULPI'     : (0xD8, 26),

    # AHB2
    'DCMI'              : (0xDC,  0),
    'CRYPT'             : (0xDC,  4),
    'HASH'              : (0xDC,  5),
    'RNG'               : (0xDC,  6),
    'SDMMC2'            : (0xDC,  9),
    'FMAC'              : (0xDC, 16),
    'CORDIC'            : (0xDC, 17),
    'SRAM1'             : (0xDC, 29),
    'SRAM2'             : (0xDC, 30),

    # AHB4
    'GPIOA'             : (0xE0,  0),
    'GPIOB'             : (0xE0,  1),
    'GPIOC'             : (0xE0,  2),
    'GPIOD'             : (0xE0,  3),
    'GPIOE'             : (0xE0,  4),
    'GPIOF'             : (0xE0,  5),
    'GPIOG'             : (0xE0,  6),
    'GPIOH'             : (0xE0,  7),
    'GPIOJ'             : (0xE0,  9),
    'GPIOK'             : (0xE0, 10),
    'CRC'               : (0xE0, 19),
    'BDMA'              : (0xE0, 21),
    'ADC3'              : (0xE0, 24),
    'HSEM'              : (0xE0, 25),
    'BKPRAM'            : (0xE0, 28),

    # APB3
    'LTDC'              : (0xE4,  3),
    'WWDG1'             : (0xE4,  6),

    # APB1L
    'TIM2'              : (0xE8,  0),
    'TIM3'              : (0xE8,  1),
    'TIM4'              : (0xE8,  2),
    'TIM5'              : (0xE8,  3),
    'TIM6'              : (0xE8,  4),
    'TIM7'              : (0xE8,  5),
    'TIM12'             : (0xE8,  6),
    'TIM13'             : (0xE8,  7),
    'TIM14'             : (0xE8,  8),
    'LPTIM1'            : (0xE8,  9),
    'SPI2'              : (0xE8, 14),
    'SPI3'              : (0xE8, 15),
    'SPDIFRX'           : (0xE8, 16),
    'USART2'            : (0xE8, 17),
    'USART3'            : (0xE8, 18),
    'UART4'             : (0xE8, 19),
    'UART5'             : (0xE8, 20),
    'I2C1'              : (0xE8, 21),
    'I2C2'              : (0xE8, 22),
    'I2C3'              : (0xE8, 23),
    'CEC'               : (0xE8, 27),
    'DAC1'              : (0xE8, 29),
    'UART7'             : (0xE8, 30),
    'UART8'             : (0xE8, 31),

    # APB1H
    'CRS'               : (0xEC,  1),
    'SWP'               : (0xEC,  2),
    'OPAMP'             : (0xEC,  4),
    'MDIOS'             : (0xEC,  5),
    'FDCAN'             : (0xEC,  8),
    'TIM23'             : (0xEC, 24),
    'TIM24'             : (0xEC, 25),

    # APB2
    'TIM1'              : (0xF0,  0),
    'TIM8'              : (0xF0,  1),
    'USART1'            : (0xF0,  4),
    'USART6'            : (0xF0,  5),
    'UART9'             : (0xF0,  6),
    'USART10'           : (0xF0,  7),
    'SPI1'              : (0xF0, 12),
    'SPI4'              : (0xF0, 13),
    'TIM15'             : (0xF0, 16),
    'TIM16'             : (0xF0, 17),
    'TIM17'             : (0xF0, 18),
    'SPI5'              : (0xF0, 20),
    'SAI1'              : (0xF0, 22),
    'DFSDM1'            : (0xF0, 30),

    # APB4
    'SYSCFG'            : (0xF4,  1),
    'LPUART1'           : (0xF4,  3),
    'SPI6'              : (0xF4,  5),
    'I2C4'              : (0xF4,  7),
    'LPTIM2'            : (0xF4,  9),
    'LPTIM3'            : (0xF4, 10),
    'LPTIM4'            : (0xF4, 11),
    'LPTIM5'            : (0xF4, 12),
    'COMP12'            : (0xF4, 14),
    'VREFEN'            : (0xF4, 15),
    'RTCAPB'            : (0xF4, 16),
    'SAI4'              : (0xF4, 21),
    'DTS'               : (0xF4, 26),
    }


class RCC_2x_3x(Device):
    '''
    Driver for STM32H72x/73x RCC device.
    '''
    REGS = [Reg32('CR',             0x000, [('HSION',               1),
                                            ('HSIKERON',            1),
                                            ('HSIRDY',              1),
                                            ('HSIDIV',              2),
                                            ('HSIDIVF',             1),
                                            ('',                    1),
                                            ('CSION',               1),
                                            ('CSIRDY',              1),
                                            ('CSIKERON',            1),
                                            ('',                    2),
                                            ('HSI48ON',             1),
                                            ('HSI48RDY',            1),
                                            ('D1CKRDY',             1),
                                            ('D2CKRDY',             1),
                                            ('HSEON',               1),
                                            ('HSERDY',              1),
                                            ('HSEBYP',              1),
                                            ('HSECSSON',            1),
                                            ('',                    4),
                                            ('PLL1ON',              1),
                                            ('PLL1RDY',             1),
                                            ('PLL2ON',              1),
                                            ('PLL2RDY',             1),
                                            ('PLL3ON',              1),
                                            ('PLL3RDY',             1),
                                            ]),
            Reg32('HSICFGR',       0x004,  [('HSICAL',              12),
                                            ('',                    12),
                                            ('HSITRIM',             7),
                                            ]),
            Reg32('CRRCR',         0x008,  [('HSI48CAL',            10),
                                            ]),
            Reg32('CSICFGR',       0x00C,  [('CSICAL',              10),
                                            ('',                    14),
                                            ('CSITRIM',             6),
                                            ]),
            Reg32('CFGR',          0x010,  [('SW',                  3),
                                            ('SWS',                 3),
                                            ('STOPWUCK',            1),
                                            ('STOPKERWUCK',         1),
                                            ('RTCPRE',              6),
                                            ('',                    1),
                                            ('TIMPRE',              1),
                                            ('',                    2),
                                            ('MCO1PRE',             4),
                                            ('MCO1',                3),
                                            ('MCO2PRE',             4),
                                            ('MCO2',                3),
                                            ]),
            Reg32('D1CFGR',        0x018,  [('HPRE',                4),
                                            ('D1PPRE',              3),
                                            ('',                    1),
                                            ('D1CPRE',              4),
                                            ]),
            Reg32('D2CFGR',        0x01C,  [('',                    4),
                                            ('D2PPRE1',             3),
                                            ('',                    1),
                                            ('D2PPRE2',             3),
                                            ]),
            Reg32('D3CFGR',        0x020,  [('',                    4),
                                            ('D3PPRE',              3),
                                            ]),
            Reg32('PLLCKSELR',     0x028,  [('PLLSRC',              2),
                                            ('',                    2),
                                            ('DIVM1',               6),
                                            ('',                    2),
                                            ('DIVM2',               6),
                                            ('',                    2),
                                            ('DIVM3',               6),
                                            ]),
            Reg32('PLLCFGR',       0x02C,  [('PLL1FRACEN',          1),
                                            ('PLL1VCOSEL',          1),
                                            ('PLL1RGE',             2),
                                            ('PLL2FRACEN',          1),
                                            ('PLL2CVOSEL',          1),
                                            ('PLL2RGE',             2),
                                            ('PLL3FRACEN',          1),
                                            ('PLL3VCOSEL',          1),
                                            ('PLL3RGE',             2),
                                            ('',                    4),
                                            ('DIVP1EN',             1),
                                            ('DIVQ1EN',             1),
                                            ('DIVR1EN',             1),
                                            ('DIVP2EN',             1),
                                            ('DIVQ2EN',             1),
                                            ('DIVR2EN',             1),
                                            ('DIVP3EN',             1),
                                            ('DIVQ3EN',             1),
                                            ('DIVR3EN',             1),
                                            ]),
            Reg32('PLL1DIVR',      0x030,  [('DIVN1',               9),
                                            ('DIVP1',               7),
                                            ('DIVQ1',               7),
                                            ('',                    1),
                                            ('DIVR1',               7),
                                            ]),
            Reg32('PLL1FRACR',     0x034,  [('',                    3),
                                            ('FRACN1',              13),
                                            ]),
            Reg32('PLL2DIVR',      0x038,  [('DIVN2',               9),
                                            ('DIVP2',               7),
                                            ('DIVQ2',               7),
                                            ('',                    1),
                                            ('DIVR2',               7),
                                            ]),
            Reg32('PLL2FRACR',     0x03C,  [('',                    3),
                                            ('FRACN1',              13),
                                            ]),
            Reg32('PLL3DIVR',      0x040,  [('DIVN3',               9),
                                            ('DIVP3',               7),
                                            ('DIVQ3',               7),
                                            ('',                    1),
                                            ('DIVR3',               7),
                                            ]),
            Reg32('PLL3FRACR',     0x044,  [('',                    3),
                                            ('FRACN3',              13),
                                            ]),
            Reg32('D1CCIPR',       0x04C,  [('FMCSEL',              2),
                                            ('',                    2),
                                            ('OCTOSPISEL',          2),
                                            ('',                    10),
                                            ('SDMMCSEL',            1),
                                            ('',                    11),
                                            ('CKPERSEL',            2),
                                            ]),
            Reg32('D2CCIP1R',      0x050,  [('SAI1SEL',             3),
                                            ('',                    9),
                                            ('SPI123SEL',           3),
                                            ('',                    1),
                                            ('SPI45SEL',            3),
                                            ('',                    1),
                                            ('SPDIFRXSEL',          2),
                                            ('',                    2),
                                            ('DFSDM1SEL',           1),
                                            ('',                    3),
                                            ('FDCANSEL',            2),
                                            ('',                    1),
                                            ('SWPMISEL',            1),
                                            ]),
            AReg32('D2CCIP2R',     0x054,  [('USART2345678SEL',     0,  2),
                                            ('USART16910SEL',       3,  5),
                                            ('RNGSEL',              8,  9),
                                            ('I2C1235SEL',          12, 13),
                                            ('USBSEL',              20, 21),
                                            ('CECSEL',              22, 23),
                                            ('LPTIM1SEL',           28, 30),
                                            ]),
            Reg32('D3CCIPR',       0x058,  [('LPUART1SEL',          3),
                                            ('',                    5),
                                            ('I2C4SEL',             2),
                                            ('LPTIM2SEL',           3),
                                            ('LPTIM345SEL',         3),
                                            ('ADCSEL',              2),
                                            ('',                    3),
                                            ('SAI4ASEL',            3),
                                            ('SAI4BSEL',            3),
                                            ('',                    1),
                                            ('SPI6SEL',             3),
                                            ]),
            Reg32('CIER',          0x060,  [('LSIRDYIE',            1),
                                            ('LSERDYIE',            1),
                                            ('HSIRDYIE',            1),
                                            ('HSERDYIE',            1),
                                            ('CSIRDYIE',            1),
                                            ('HSI48RDYIE',          1),
                                            ('PLL1RDYIE',           1),
                                            ('PLL2RDYIE',           1),
                                            ('PLL3RDYIE',           1),
                                            ('LSECSSIE',            1),
                                            ]),
            Reg32('CIFR',          0x064,  [('LSIRDYF',             1),
                                            ('LSERDYF',             1),
                                            ('HSIRDYF',             1),
                                            ('HSERDYF',             1),
                                            ('CSIRDYF',             1),
                                            ('HSI48RDYF',           1),
                                            ('PLL1RDYF',            1),
                                            ('PLL2RDYF',            1),
                                            ('PLL3RDYF',            1),
                                            ('LSECSSF',             1),
                                            ('HSECSSF',             1),
                                            ]),
            Reg32('CICR',          0x068,  [('LSIRDYC',             1),
                                            ('LSERDYC',             1),
                                            ('HSIRDYC',             1),
                                            ('HSERDYC',             1),
                                            ('CSIRDYC',             1),
                                            ('HSI48RDYC',           1),
                                            ('PLL1RDYC',            1),
                                            ('PLL2RDYC',            1),
                                            ('PLL3RDYC',            1),
                                            ('LSECSSC',             1),
                                            ('HSECSSC',             1),
                                            ]),
            Reg32('BDCR',          0x070,  [('LSEON',               1),
                                            ('LSERDY',              1),
                                            ('LSEBYP',              1),
                                            ('LSEDRV',              2),
                                            ('LSECSSON',            1),
                                            ('LSECSSD',             1),
                                            ('',                    1),
                                            ('RTCSEL',              2),
                                            ('',                    5),
                                            ('RTCEN',               1),
                                            ('BDRST',               1),
                                            ]),
            Reg32('CSR',           0x074,  [('LSION',               1),
                                            ('LSIRDY',              1),
                                            ]),
            Reg32('AHB3RSTR',      0x07C,  [('MDMARST',             1),
                                            ('',                    3),
                                            ('DMA2DRST',            1),
                                            ('',                    7),
                                            ('FMCRST',              1),
                                            ('',                    1),
                                            ('OCTOSPI1RST',         1),
                                            ('',                    1),
                                            ('SDMMC1RST',           1),
                                            ('',                    2),
                                            ('OCTOSPI2RST',         1),
                                            ('',                    1),
                                            ('IOMNGRRST',           1),
                                            ('OTFD1RST',            1),
                                            ('OTFD2RST',            1),
                                            ('',                    7),
                                            ('CPURST',              1),
                                            ]),
            Reg32('AHB1RSTR',      0x080,  [('DMA1RST',             1),
                                            ('DMA2RST',             1),
                                            ('',                    3),
                                            ('ADC12RST',            1),
                                            ('',                    9),
                                            ('ETH1MACRST',          1),
                                            ('',                    9),
                                            ('USB1OTGRST',          1),
                                            ]),
            Reg32('AHB2RSTR',      0x084,  [('DCMI_PSSIRST',        1),
                                            ('',                    3),
                                            ('CRYPTRST',            1),
                                            ('HASHRST',             1),
                                            ('RNGRST',              1),
                                            ('',                    2),
                                            ('SDMMC2RST',           1),
                                            ('',                    6),
                                            ('FMACRST',             1),
                                            ('CORDICRST',           1),
                                            ]),
            Reg32('AHB4RSTR',      0x088,  [('GPIOARST',            1),
                                            ('GPIOBRST',            1),
                                            ('GPIOCRST',            1),
                                            ('GPIODRST',            1),
                                            ('GPIOERST',            1),
                                            ('GPIOFRST',            1),
                                            ('GPIOGRST',            1),
                                            ('GPIOHRST',            1),
                                            ('',                    1),
                                            ('GPIOJRST',            1),
                                            ('GPIOKRST',            1),
                                            ('',                    8),
                                            ('CRCRST',              1),
                                            ('',                    1),
                                            ('BDMARST',             1),
                                            ('',                    2),
                                            ('ADC3RST',             1),
                                            ('HSEMRST',             1),
                                            ]),
            Reg32('APB3RSTR',      0x08C,  [('',                    3),
                                            ('LTDCRST',             1),
                                            ]),
            Reg32('APB1LRSTR',     0x090,  [('TIM2RST',             1),
                                            ('TIM3RST',             1),
                                            ('TIM4RST',             1),
                                            ('TIM5RST',             1),
                                            ('TIM6RST',             1),
                                            ('TIM7RST',             1),
                                            ('TIM12RST',            1),
                                            ('TIM13RST',            1),
                                            ('TIM14RST',            1),
                                            ('LPTIM1RST',           1),
                                            ('',                    4),
                                            ('SPI2RST',             1),
                                            ('SPI3RST',             1),
                                            ('SPDIFRXRST',          1),
                                            ('USART2RST',           1),
                                            ('USART3RST',           1),
                                            ('UART4RST',            1),
                                            ('UART5RST',            1),
                                            ('I2C1RST',             1),
                                            ('I2C2RST',             1),
                                            ('I2C3RST',             1),
                                            ('',                    1),
                                            ('I2C5RST',             1),
                                            ('',                    1),
                                            ('CECRST',              1),
                                            ('',                    1),
                                            ('DAC1RST',             1),
                                            ('UART7RST',            1),
                                            ('UART8RST',            1),
                                            ]),
            Reg32('APB1HRSTR',     0x094,  [('',                    1),
                                            ('CRSRST',              1),
                                            ('SWPMPRST',            1),
                                            ('',                    1),
                                            ('OPAMPRST',            1),
                                            ('MDIOSRST',            1),
                                            ('',                    2),
                                            ('FDCANRST',            1),
                                            ('',                    15),
                                            ('TIM23RST',            1),
                                            ('TIM24RST',            1),
                                            ]),
            Reg32('APB2RSTR',      0x098,  [('TIM1RST',             1),
                                            ('TIM8RST',             1),
                                            ('',                    2),
                                            ('USART1RST',           1),
                                            ('USART6RST',           1),
                                            ('UART9RST',            1),
                                            ('UART10RST',           1),
                                            ('',                    4),
                                            ('SPI1RST',             1),
                                            ('SPI4RST',             1),
                                            ('',                    2),
                                            ('TIM15RST',            1),
                                            ('TIM16RST',            1),
                                            ('TIM17RST',            1),
                                            ('',                    1),
                                            ('SPI5RST',             1),
                                            ('',                    1),
                                            ('SAI1RST',             1),
                                            ('',                    7),
                                            ('DFSDM1RST',           1),
                                            ]),
            Reg32('APB4RSTR',      0x09C,  [('',                    1),
                                            ('SYSCFGRST',           1),
                                            ('',                    1),
                                            ('LPUART1RST',          1),
                                            ('',                    1),
                                            ('SPI6RST',             1),
                                            ('',                    1),
                                            ('I2C4RST',             1),
                                            ('',                    1),
                                            ('LPTIM2RST',           1),
                                            ('LPTIM3RST',           1),
                                            ('LPTIM4RST',           1),
                                            ('LPTIM5RST',           1),
                                            ('',                    1),
                                            ('COMP12RST',           1),
                                            ('VREFRST',             1),
                                            ('',                    5),
                                            ('SAI4RST',             1),
                                            ('',                    4),
                                            ('DTSRST',              1),
                                            ]),
            Reg32('GCR',           0x0A0,  [('WW1RSC',              1),
                                            ]),
            Reg32('D3AMR',         0x0A8,  [('BDMAAMEN',            1),
                                            ('',                    2),
                                            ('LPUART1AMEN',         1),
                                            ('',                    1),
                                            ('SPI6AMEN',            1),
                                            ('',                    1),
                                            ('I2C4AMEN',            1),
                                            ('',                    1),
                                            ('LPTIM2AMEN',          1),
                                            ('LPTIM3AMEN',          1),
                                            ('LPTIM4AMEN',          1),
                                            ('LPTIM5AMEN',          1),
                                            ('',                    1),
                                            ('COMP12AMEN',          1),
                                            ('VREFAMEN',            1),
                                            ('RTCAMEN',             1),
                                            ('',                    2),
                                            ('CRCAMEN',             1),
                                            ('',                    1),
                                            ('SAI4AMEN',            1),
                                            ('',                    2),
                                            ('ADC3AMEN',            1),
                                            ('',                    1),
                                            ('DTSAMEN',             1),
                                            ('',                    1),
                                            ('BKPRAMAMEN',          1),
                                            ('SRAM4AMEN',           1),
                                            ]),
            Reg32('RSR',           0x0D0,  [('',                    16),
                                            ('RMVF',                1),
                                            ('CPURSTF',             1),
                                            ('',                    1),
                                            ('D1RSTF',              1),
                                            ('D2RSTF',              1),
                                            ('BORRSTF',             1),
                                            ('PINRSTF',             1),
                                            ('PORRSTF',             1),
                                            ('SFTRSTF',             1),
                                            ('',                    1),
                                            ('IWDG1RSTF',           1),
                                            ('',                    1),
                                            ('WWDG1RSTF',           1),
                                            ('',                    1),
                                            ('LPWRRSTF',            1),
                                            ]),
            Reg32('AHB3ENR',       0x0D4,  [('MDMAEN',              1),
                                            ('',                    3),
                                            ('DMA2DEN',             1),
                                            ('',                    7),
                                            ('FMCEN',               1),
                                            ('',                    1),
                                            ('OCTOSPI1EN',          1),
                                            ('',                    1),
                                            ('SDMMC1EN',            1),
                                            ('',                    2),
                                            ('OCTOSPI2EN',          1),
                                            ('',                    1),
                                            ('IOMNGREN',            1),
                                            ('OTFD1EN',             1),
                                            ('OTFD2EN',             1),
                                            ]),
            Reg32('AHB1ENR',       0x0D8,  [('DMA1EN',              1),
                                            ('DMA2EN',              1),
                                            ('',                    3),
                                            ('ADC12EN',             1),
                                            ('',                    9),
                                            ('ETH1MACEN',           1),
                                            ('ETH1TXEN',            1),
                                            ('ETH1RXEN',            1),
                                            ('',                    7),
                                            ('USB1OTGEN',           1),
                                            ('USB1OTGHSULPIEN',     1),
                                            ]),
            Reg32('AHB2ENR',       0x0DC,  [('DCMI_PSSI',           1),
                                            ('',                    3),
                                            ('CRYPTEN',             1),
                                            ('HASHEN',              1),
                                            ('RNGEN',               1),
                                            ('',                    2),
                                            ('SDMMC2EN',            1),
                                            ('',                    6),
                                            ('FMACEN',              1),
                                            ('CORDICEN',            1),
                                            ('',                    11),
                                            ('SRAM1EN',             1),
                                            ('SRAM2EN',             1),
                                            ]),
            Reg32('AHB4ENR',       0x0E0,  [('GPIOAEN',             1),
                                            ('GPIOBEN',             1),
                                            ('GPIOCEN',             1),
                                            ('GPIODEN',             1),
                                            ('GPIOEEN',             1),
                                            ('GPIOFEN',             1),
                                            ('GPIOGEN',             1),
                                            ('GPIOHEN',             1),
                                            ('',                    1),
                                            ('GPIOJEN',             1),
                                            ('GPIOKEN',             1),
                                            ('',                    8),
                                            ('CRCEN',               1),
                                            ('',                    1),
                                            ('BDMAEN',              1),
                                            ('',                    2),
                                            ('ADC3EN',              1),
                                            ('HSEMEN',              1),
                                            ('',                    2),
                                            ('BKPRAMEN',            1),
                                            ]),
            Reg32('APB3ENR',       0x0E4,  [('',                    3),
                                            ('LTDCEN',              1),
                                            ('',                    2),
                                            ('WWDG1EN',             1),
                                            ]),
            Reg32('APB1LENR',      0x0E8,  [('TIM2EN',              1),
                                            ('TIM3EN',              1),
                                            ('TIM4EN',              1),
                                            ('TIM5EN',              1),
                                            ('TIM6EN',              1),
                                            ('TIM7EN',              1),
                                            ('TIM12EN',             1),
                                            ('TIM13EN',             1),
                                            ('TIM14EN',             1),
                                            ('LPTIM1EN',            1),
                                            ('',                    4),
                                            ('SPI2EN',              1),
                                            ('SPI3EN',              1),
                                            ('SPDIFRXEN',           1),
                                            ('USART2EN',            1),
                                            ('USART3EN',            1),
                                            ('UART4EN',             1),
                                            ('UART5EN',             1),
                                            ('I2C1EN',              1),
                                            ('I2C2EN',              1),
                                            ('I2C3EN',              1),
                                            ('',                    1),
                                            ('I2C5EN',              1),
                                            ('',                    1),
                                            ('CECEN',               1),
                                            ('',                    1),
                                            ('DAC1EN',              1),
                                            ('UART7EN',             1),
                                            ('UART8EN',             1),
                                            ]),
            Reg32('APB1HENR',      0x0EC,  [('',                    1),
                                            ('CRSEN',               1),
                                            ('SWPMIEN',             1),
                                            ('',                    1),
                                            ('OPAMPEN',             1),
                                            ('MDIOSEN',             1),
                                            ('',                    2),
                                            ('FDCANEN',             1),
                                            ('',                    15),
                                            ('TIM23EN',             1),
                                            ('TIM24EN',             1),
                                            ]),
            Reg32('APB2ENR',       0x0F0,  [('TIM1EN',              1),
                                            ('TIM8EN',              1),
                                            ('',                    2),
                                            ('USART1EN',            1),
                                            ('USART6EN',            1),
                                            ('UART9EN',             1),
                                            ('USART10EN',           1),
                                            ('',                    4),
                                            ('SPI1EN',              1),
                                            ('SPI4EN',              1),
                                            ('',                    2),
                                            ('TIM15EN',             1),
                                            ('TIM16EN',             1),
                                            ('TIM17EN',             1),
                                            ('',                    1),
                                            ('SPI5EN',              1),
                                            ('',                    1),
                                            ('SAI1EN',              1),
                                            ('',                    7),
                                            ('DFSDM1EN',            1),
                                            ]),
            Reg32('APB4ENR',       0x0F4,  [('',                    1),
                                            ('SYSCFGEN',            1),
                                            ('',                    1),
                                            ('LPUART1EN',           1),
                                            ('',                    1),
                                            ('SPI6EN',              1),
                                            ('',                    1),
                                            ('I2C4EN',              1),
                                            ('',                    1),
                                            ('LPTIM2EN',            1),
                                            ('LPTIM3EN',            1),
                                            ('LPTIM4EN',            1),
                                            ('LPTIM5EN',            1),
                                            ('',                    1),
                                            ('COMP12EN',            1),
                                            ('VREFEN',              1),
                                            ('RTCAPBEN',            1),
                                            ('',                    4),
                                            ('SAI4EN',              1),
                                            ('',                    4),
                                            ('DTSEN',               1),
                                            ]),
            Reg32('AHB3LPENR',     0x0FC,  [('MDMALPEN',            1),
                                            ('',                    3),
                                            ('DMA2DLPEN',           1),
                                            ('',                    3),
                                            ('FLASHLPEN',           1),
                                            ('',                    3),
                                            ('FMCLPEN',             1),
                                            ('',                    1),
                                            ('OCTO1LPEN',           1),
                                            ('',                    1),
                                            ('SDMMC1LPEN',          1),
                                            ('',                    2),
                                            ('OCTO2LPEN',           1),
                                            ('',                    1),
                                            ('IOMNGRLPEN',          1),
                                            ('OTFD1LPEN',           1),
                                            ('OTFD2LPEN',           1),
                                            ('',                    4),
                                            ('DTCM1LPEN',           1),
                                            ('DTCM2LPEN',           1),
                                            ('ITCMLPEN',            1),
                                            ('AXISRAMLPEN',         1),
                                            ]),
            Reg32('AHB1LPENR',     0x100,  [('DMA1LPEN',            1),
                                            ('DMA2LPEN',            1),
                                            ('',                    3),
                                            ('ADC12LPEN',           1),
                                            ('',                    9),
                                            ('ETH1MACLPEN',         1),
                                            ('ETH1TXLPEN',          1),
                                            ('ETH1RXLPEN',          1),
                                            ('',                    7),
                                            ('USB1OTGLPEN',         1),
                                            ('USB1OTGHSULPILPEN',   1),
                                            ]),
            Reg32('AHB2LPENR',     0x104,  [('DCMI_PSSILPEN',       1),
                                            ('',                    3),
                                            ('CRYPTLPEN',           1),
                                            ('HASHLPEN',            1),
                                            ('RNGLPEN',             1),
                                            ('',                    2),
                                            ('SDMMC2LPEN',          1),
                                            ('',                    6),
                                            ('FMACLPEN',            1),
                                            ('CORDICLPEN',          1),
                                            ('',                    11),
                                            ('SRAM1LPEN',           1),
                                            ('SRAM2LPEN',           1),
                                            ]),
            Reg32('AHB4LPENR',     0x108,  [('GPIOALPEN',           1),
                                            ('GPIOBLPEN',           1),
                                            ('GPIOCLPEN',           1),
                                            ('GPIODLPEN',           1),
                                            ('GPIOELPEN',           1),
                                            ('GPIOFLPEN',           1),
                                            ('GPIOGLPEN',           1),
                                            ('GPIOHLPEN',           1),
                                            ('',                    1),
                                            ('GPIOJLPEN',           1),
                                            ('GPIOKLPEN',           1),
                                            ('',                    8),
                                            ('CRCLPEN',             1),
                                            ('',                    1),
                                            ('BDMALPEN',            1),
                                            ('',                    2),
                                            ('ADC3LPEN',            1),
                                            ('',                    3),
                                            ('BKPRAMLPEN',          1),
                                            ('SRAM4LPEN',           1),
                                            ]),
            Reg32('APB3LPENR',     0x10C,  [('',                    3),
                                            ('LTDCLPEN',            1),
                                            ('',                    2),
                                            ('WWDG1LPEN',           1),
                                            ]),
            Reg32('APB1LLPENR',    0x110,  [('TIM2LPEN',            1),
                                            ('TIM3LPEN',            1),
                                            ('TIM4LPEN',            1),
                                            ('TIM5LPEN',            1),
                                            ('TIM6LPEN',            1),
                                            ('TIM7LPEN',            1),
                                            ('TIM12LPEN',           1),
                                            ('TIM13LPEN',           1),
                                            ('TIM14LPEN',           1),
                                            ('LPTIM1LPEN',          1),
                                            ('',                    4),
                                            ('SPI2LPEN',            1),
                                            ('SPI3LPEN',            1),
                                            ('SPDIFRXLPEN',         1),
                                            ('USART2LPEN',          1),
                                            ('USART3LPEN',          1),
                                            ('UART4LPEN',           1),
                                            ('UART5LPEN',           1),
                                            ('I2C1LPEN',            1),
                                            ('I2C2LPEN',            1),
                                            ('I2C3LPEN',            1),
                                            ('',                    1),
                                            ('I2C5LPEN',            1),
                                            ('',                    1),
                                            ('CECLPEN',             1),
                                            ('',                    1),
                                            ('DAC1LPEN',            1),
                                            ('UART7LPEN',           1),
                                            ('UART8LPEN',           1),
                                            ]),
            Reg32('APB1HLPENR',    0x114,  [('',                    1),
                                            ('CRSLPEN',             1),
                                            ('SWPMILPEN',           1),
                                            ('',                    1),
                                            ('OPAMPLPEN',           1),
                                            ('MDIOSLPEN',           1),
                                            ('',                    2),
                                            ('FDCANLPEN',           1),
                                            ('',                    15),
                                            ('TIM23LPEN',           1),
                                            ('TIM24LPEN',           1),
                                            ]),
            Reg32('APB2LPENR',     0x118,  [('TIM1LPEN',            1),
                                            ('TIM8LPEN',            1),
                                            ('',                    2),
                                            ('USART1LPEN',          1),
                                            ('USART6LPEN',          1),
                                            ('UART9LPEN',           1),
                                            ('USART10LPEN',         1),
                                            ('',                    4),
                                            ('SPI1LPEN',            1),
                                            ('SPI4LPEN',            1),
                                            ('',                    2),
                                            ('TIM15LPEN',           1),
                                            ('TIM16LPEN',           1),
                                            ('TIM17LPEN',           1),
                                            ('',                    1),
                                            ('SPI5LPEN',            1),
                                            ('',                    1),
                                            ('SAI1LPEN',            1),
                                            ('',                    7),
                                            ('DFSDM1LPEN',          1),
                                            ]),
            Reg32('APB4LPENR',     0x11C,  [('',                    1),
                                            ('SYSCFGLPEN',          1),
                                            ('',                    1),
                                            ('LPUART1LPEN',         1),
                                            ('',                    1),
                                            ('SPI6LPEN',            1),
                                            ('',                    1),
                                            ('I2C4LPEN',            1),
                                            ('',                    1),
                                            ('LPTIM2LPEN',          1),
                                            ('LPTIM3LPEN',          1),
                                            ('LPTIM4LPEN',          1),
                                            ('LPTIM5LPEN',          1),
                                            ('',                    1),
                                            ('COMP12LPEN',          1),
                                            ('VREFLPEN',            1),
                                            ('RTCAPBLPEN',          1),
                                            ('',                    4),
                                            ('SAI4LPEN',            1),
                                            ('',                    4),
                                            ('DTSLPEN',             1),
                                            ]),
            ]

    def __init__(self, target, ap, name, addr, **kwargs):
        super().__init__(target, ap, addr, name, RCC_2x_3x.REGS, **kwargs)
